-------------------------------------------------------------------------------
-- Name: class ActivityLog
-- Author: Jonathan Primeau
--
-- Description:
--     This class takes care of the activity logs.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pFileObj              -- class File script instance
property pFilename             -- Log filename
property pTimeStampFlag        -- Time stamp logs flag
property pCreateLogFlag        -- Create log file flag
property pFlashConnector
property pPolicy


-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void SetFilename (me, theFilename)
--
-- This method stores the log filename in its property.
--
-- Parameters:
--     me (object): this pointer.
--     theFilename (string): The log filename
--
-------------------------------------------------------------------------------
on SetFilename (me, theFilename)
  me.pFilename = theFilename
end

-------------------------------------------------------------------------------
-- void SetCreateFlag (me, theCreateFlag)
--
-- This method stores the create flag in its property. If set to true the log
-- file will be created and thus deleted if it already exists.
--
-- Parameters:
--     me (object): this pointer.
--     theCreateFlag (bool): Creates the log file if true
--
-------------------------------------------------------------------------------
on SetCreateFlag (me, theCreateFlag)
  me.pCreateLogFlag = theCreateFlag
end

on SetPolicy (me, thePolicy)
  me.pPolicy = thePolicy
end

-------------------------------------------------------------------------------
-- void Log (me, theString)
--
-- This method logs the specified string in a new log entry. According to the
-- preset properties, it might create the log first and/or log the time stamp.
--
-- Parameters:
--     me (object): this pointer.
--     theString (bool): The string to log.
--
-------------------------------------------------------------------------------
on Log (me, theType, theSubType, theDetail)
  lSubTypeValue = string(theSubType)
  lDetailValue = string(theDetail)
  
  -- Validate theType
  lTypeValue = "Unknown"
  case (theType) of
    #operational:
      lTypeValue = "operational"
    #navigation:
      lTypeValue = "navigation"
    #patron:
      lTypeValue = "patron"
    #content:
      lTypeValue = "content"
    #access:
      lTypeValue = "access"
    #default:
      GenLog(#error, "ActivityLog::Log: Unknown activity type!")
      lTypeValue = "Unknown"
  end case
  
  -- Build the XML (The OPS doesn't seem to be able to parse normal XML...)
  lXML = me.pFlashConnector.newObject("XML")
  
  lActLog = lXML.createElement("activity_log")
  lXML.appendChild(lActLog)
  
  lKioskID = lXML.createElement("kiosk_id")
  lTextNode = lXML.createTextNode(me._GetKioskID())
  lKioskID.appendChild(lTextNode)
  lActLog.appendChild(lKioskID)
  
  lKioskName = lXML.createElement("kiosk_name")
  lTextNode = lXML.createTextNode(me._GetKioskName())
  lKioskName.appendChild(lTextNode)
  lActLog.appendChild(lKioskName)
  
  lPlayerID = lXML.createElement("player_id")
  lTextNode = lXML.createTextNode(me._GetPlayerID())
  lPlayerID.appendChild(lTextNode)
  lActLog.appendChild(lPlayerID)
  
  lTime = lXML.createElement("time")
  lTextNode = lXML.createTextNode(me._GetTime())
  lTime.appendChild(lTextNode)
  lActLog.appendChild(lTime)
  
  lType = lXML.createElement("type")
  lTextNode = lXML.createTextNode(lTypeValue)
  lType.appendChild(lTextNode)
  lActLog.appendChild(lType)
  
  if (lSubTypeValue <> "") then
    lSubType = lXML.createElement("sub_type")
    lTextNode = lXML.createTextNode(lSubTypeValue)
    lSubType.appendChild(lTextNode)
    lActLog.appendChild(lSubType)
  end if
  
  if (lDetailValue <> "") then
    lDetail = lXML.createElement("detail")
    lTextNode = lXML.createTextNode(lDetailValue)
    lDetail.appendChild(lTextNode)
    lActLog.appendChild(lDetail)
  end if
  
  lData = lXML.toString()
  
  -- If the policy is "server", send activity to the OPS
  if (me.pPolicy = "server") then
    me._SendLog(lData)
  end if
  
  -- Always log activities on disk
  me._WriteLog(lData)
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pFileObj = script("class File").new()
  me.pFilename = ""
  me.pCreateLogFlag = false
  me.pFlashConnector = sprite("Terminal")
  me.pPolicy = "disk"
end

on _GetKioskID (me)
  lKioskID = QI(#status).GetKioskID()
  return(lKioskID)
end

on _GetKioskName (me)
  lKioskName = QI(#status).GetKioskName()  
  return(lKioskName)
end

on _GetPlayerID (me)
  lPlayerID = 0
  lPatronInfo = QI(#patron).GetPatronInfo()
  if (listP(lPatronInfo)) then
    if (lPatronInfo.getaProp(#player) <> void) then
      lPlayerID = lPatronInfo.player
    end if
  end if
  return(lPlayerID)
end

on _GetTime (me)
  lTime = GetLogTimeStamp()
  return(lTime)
end


-------------------------------------------------------------------------------
-- void _CreateLogFile (me)
--
-- Creates the log file. It deletes the old one if it already exists.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _CreateLogFile (me)
  -- Make sure the file is deleted
  me.pFileObj.DeleteFile(me.pFilename)
  -- Create the log file
  me.pFileObj.CreateFile(me.pFilename)
end

on _WriteLog (me, theData)
  -- Create the file if needed
  if (me.pCreateLogFlag or me.pFileObj.FileExists(me.pFilename) = 0) then
    me._CreateLogFile()
    me.pCreateLogFlag = 0
  end if
  
  -- Open the file using write mode and go to the end
  me.pFileObj.OpenFile(me.pFilename, #write)
  me.pFileObj.SetPositionEOF()
  
  -- Write the data
  me.pFileObj.WriteString(theData)
  me.pFileObj.WriteReturn()
  
  -- Close the file
  me.pFileObj.CloseFile()
end

on _SendLog (me, theData)
  if (QI(#opsCom).IsConnected()) then
    QI(#opsCom).LogActivity(theData)
  end if
end