-------------------------------------------------------------------------------
-- Name: class Download
-- Author: Jonathan Primeau
--
-- Description:
--     This class offers an API for downloading stuff from an URL
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pCache                -- List of downloaded items in cache
property pNetID                -- Current download ID
property pURL                  -- Current URL for download (source)
property pFilename             -- Current filename (destination)
property pCallback             -- Current callback information
property pStatus               -- The download status


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void Download (me)
--
-- This methods is called when the download is successful. It stops the check
-- download status timeout, adds the download to the cache and calls the 
-- callback specified when the download starts.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Download (me, theURL, theFilename, theCallback, theObject)
  
  -- TODO: Validate parameters
  
  -- Keep track of parameters for later use
  me.pStatus = 0
  me.pURL = theURL
  me.pFilename = theFilename
  me.pCallback = [#handler: theCallback, #object: theObject]
  
  -- TODO: Check in cache if we already downloaded
  if (me._CacheLookup(theURL, theFilename)) then
    -- We don't have to download the file since it's already there
    me._CallCallback()
  end if
  
  -- Start downloading...
  GenLog(#debug, "Download::Download: Downloading from" && theURL && "to" && theFilename)
  me.pNetID = downLoadNetThing(theURL, theFilename)
  
  -- Start the check download status timeout
  QI(#downloadTimeout).Start()
end

-------------------------------------------------------------------------------
-- void DoneDownload (me)
--
-- This methods is called when the download is successful. It stops the check
-- download status timeout, adds the download to the cache and calls the 
-- callback specified when the download starts.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on DoneDownload (me)
  -- Stop the check download status timeout
  QI(#downloadTimeout).Stop()
  
  -- Add to cache
  me._AddToCache(me.pURL, me.pFilename)
  
  GenLog(#debug, "Download::DoneDownload: Done downloading!")
  
  -- Done! Give the hand to our caller
  me._CallCallback()
end

-------------------------------------------------------------------------------
-- void DownloadFailed (me)
--
-- This methods is called when the download failed. It simply calls the 
-- callback specified when the download starts.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on DownloadFailed (me)
  -- Stop the check download status timeout
  QI(#downloadTimeout).Stop()
  
  GenLog(#error, "Download::DownloadFailed: Download failed NetError:" && me.pStatus)
  
  -- Failed! Give the hand to our caller
  me._CallCallback()
end

-------------------------------------------------------------------------------
-- void CheckDownloadDone (me)
--
-- Checks if the current download is done. This is called by 
-- CheckDownloadStatus periodically.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if the download is done.
--
-------------------------------------------------------------------------------
on CheckDownloadDone (me)
  if (NetDone(me.pNetID)) then
    me.DownloadDone()
    return(true)
  end if
  return(false)
end

-------------------------------------------------------------------------------
-- void CheckDownloadStatus (me)
--
-- Checks if the current download status is OK.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if the download is done.
--
-------------------------------------------------------------------------------
on CheckDownloadStatus (me)
  lStatus = NetError(me.pNetID)
  me.pStatus = lStatus
  if (lStatus <> 0) then
    me.DownloadFailed()
  end if
  return(lStatus)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pCache = []
  me.pNetID = 0
  me.pURL = ""
  me.pFilename = ""
  me.pCallback = [#handler: void, #object: void]
end

-------------------------------------------------------------------------------
-- void _CallCallback (me)
--
-- Calls the callback specified when the download starts. The same callback is
-- called whether the downloaded succeeded or not. However, the callback 
-- handler should expect some parameters. The first one is the download status
-- (0 for success and different than 0 for error).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _CallCallback (me)
  -- Call the specified callback
  lHandler = me.pCallback.handler
  lObject = me.pCallback.object
  lStatus = me.pStatus
  
  if (symbolP(lHandler) = 0) then
    ShowMessage("Callback handler is invalid.")
  end if
  
  if (objectP(lObject) = 0) then
    ShowMessage("Callback object is invalid.")
  end if
  
  call(lHandler, lObject, lStatus)
end

-------------------------------------------------------------------------------
-- void _CacheLookup (me)
--
-- This method looks through the cache to find a match and return true if it's
-- the case. This is useful to know if a file has been already downloaded.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if the requested URL and filename are already in cache.
--
-------------------------------------------------------------------------------
on _CacheLookup (me, theURL, theFilename)
  if (listP(me.pCache)) then
    lCount = me.pCache.count
    if (lCount > 0) then
      repeat with i = 1 to lCount
        -- Does it match?
        if (me.pCache[i].url = theURL and me.pCache[i].filename = theFilename) then
          -- Also check if the file exists
          if (QI(#file).FileExists(theFilename)) then
            GenLog(#debug, "Download::_CacheLookup: Found in cache")
            return(true)
          else
            -- Big problem: the cache is corrupted so invalidate the entry
            GenLog(#error, "Download::_CacheLookup: Cache corrupted URL:" && theURL && "Filename:" && theFilename)
            me.pCache[i].filename = ""
          end if
        end if
      end repeat
    end if
  end if
  return(false)
end

-------------------------------------------------------------------------------
-- void _AddToCache (me)
--
-- This method adds the specified URL and filename to the cache. This is done
-- when a download succeeded
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _AddToCache (me, theURL, theFilename)
  GenLog(#debug, "Download::_AddToCache:"&&theURL&&"added to cache.")
  me.pCache.add([#url: theURL, #filename: theFilename])
end
