-------------------------------------------------------------------------------
-- Name: class Log
-- Author: Jonathan Primeau
--
-- Description:
--     This class is meant to manage any log. It is easy to use since you only
--     have to call Log everytime you want to log something
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pFileObj              -- class File script instance
property pFilename             -- Log filename
property pTimeStampFlag        -- Time stamp logs flag
property pCreateLogFlag        -- Create log file flag
property pLogStack


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.pFileObj)) then
    me._Commit(true)
    me.pFileObj = void
  end if
  
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void SetFilename (me, theFilename)
--
-- This method stores the log filename in its property.
--
-- Parameters:
--     me (object): this pointer.
--     theFilename (string): The log filename
--
-------------------------------------------------------------------------------
on SetFilename (me, theFilename)
  me.pFilename = theFilename
end

-------------------------------------------------------------------------------
-- void SetCreateFlag (me, theCreateFlag)
--
-- This method stores the create flag in its property. If set to true the log
-- file will be created and thus deleted if it already exists.
--
-- Parameters:
--     me (object): this pointer.
--     theCreateFlag (bool): Creates the log file if true
--
-------------------------------------------------------------------------------
on SetCreateFlag (me, theCreateFlag)
  me.pCreateLogFlag = theCreateFlag
end

-------------------------------------------------------------------------------
-- void SetCreateFlag (me, theTimeFlag)
--
-- This method stores the time stamp flag in its property. If set to true it 
-- will log the time stamp for each log entry.
--
-- Parameters:
--     me (object): this pointer.
--     theTimeFlag (bool): Log the time stamp.
--
-------------------------------------------------------------------------------
on SetTimeStampFlag (me, theTimeFlag)
  me.pTimeStampFlag = theTimeFlag
end

-------------------------------------------------------------------------------
-- void Log (me, theString)
--
-- This method logs the specified string in a new log entry. According to the
-- preset properties, it might create the log first and/or log the time stamp.
--
-- Parameters:
--     me (object): this pointer.
--     theString (bool): The string to log.
--
-------------------------------------------------------------------------------
on Log (me, theLevel, theString)
  
  lLogLevel = QI(#config).GetLogLevel()
  lDoLog = false
  lPrefix = ""
  if (theLevel = #error and (lLogLevel = #error or lLogLevel = #debug or lLogLevel = #info or lLogLevel = #debugnav)) then
    lDoLog = true
    lPrefix = "[ERROR] "
  else if (theLevel = #info and (lLogLevel = #info or lLogLevel = #debug or lLogLevel = #debugnav)) then
    lDoLog = true
    lPrefix = "[INFO] "
  else if (theLevel = #debug and (lLogLevel = #debug or lLogLevel = #debugnav)) then
    lDoLog = true
    lPrefix = "[DEBUG] "
  else if (theLevel = #debugnav and (lLogLevel = #debugnav)) then
    lDoLog = true
    lPrefix = "[DEBUGNAV] "
  else
    return(lDoLog)
  end if
  
  -- Write the time stamp
  lEntry = ""
  lTimeStamp = ""
  if (me.pTimeStampFlag) then
    lTimeStamp = "[" & GetLogTimeStamp() & "] "
  end if
  
  -- Stack the log entry
  lEntry = lTimeStamp & lPrefix & theString
  me.pLogStack.add(lEntry)
  
  -- Check if commit needed
  me._Commit()
  
  return(lDoLog)
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pFileObj = script("class File").new()
  me.pFilename = ""
  me.pTimeStampFlag = true
  me.pCreateLogFlag = false
  me.pLogStack = []
end

-------------------------------------------------------------------------------
-- void _CreateLogFile (me)
--
-- Creates the log file. It deletes the old one if it already exists.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _CreateLogFile (me)
  -- Make sure the file is deleted
  me.pFileObj.DeleteFile(me.pFilename)
  -- Create the log file
  me.pFileObj.CreateFile(me.pFilename)
end

on _Commit (me, theForce)
  lForce = theForce
  if (lForce = void) then lForce = false
  
  lCount = me.pLogStack.count
  
  -- Commit only if there is 5 log entries in the stack
  -- or if we force it
  if (lCount >= 5 or (lForce = true and lCount > 0)) then
    
    -- Create the file if needed
    if (me.pCreateLogFlag or me.pFileObj.FileExists(me.pFilename) = 0) then
      me._CreateLogFile()
      me.pCreateLogFlag = 0
    end if
    
    -- Open the file using write mode
    me.pFileObj.OpenFile(me.pFilename, #write)
    
    -- Go to the end
    me.pFileObj.SetPositionEOF()
    
    repeat with i = 1 to lCount
      lCurrEntry = me.pLogStack[i]
      -- Write log entry in the file
      me.pFileObj.WriteString(lCurrEntry)
      me.pFileObj.WriteReturn()
    end repeat
    
    -- Close the file
    me.pFileObj.CloseFile()
    
    -- Empty the stack
    me.pLogStack = []
  end if
end